@@ -1,13 +1,13 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2007 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2007,2008 -- leonerd@leonerd.org.uk
 
 package IO::Async::ChildManager;
 
 use strict;
 
-our $VERSION = '0.14_2';
+our $VERSION = '0.16.001';
 
 # Not a notifier
 
@@ -15,8 +15,10 @@ use IO::Async::Stream;
 use IO::Async::MergePoint;
 
 use Carp;
+use Scalar::Util qw( weaken );
+
 use Fcntl qw( F_GETFL F_SETFL FD_CLOEXEC );
-use POSIX qw( WNOHANG _exit sysconf _SC_OPEN_MAX dup2 );
+use POSIX qw( WNOHANG _exit sysconf _SC_OPEN_MAX dup2 nice );
 
 use constant LENGTH_OF_I => length( pack( "I", 0 ) );
 
@@ -32,10 +34,8 @@ C<IO::Async::ChildManager> - facilitates the execution of child processes
 
 This object is used indirectly via an C<IO::Async::Loop>:
 
- use IO::Async::Loop::IO_Poll;
- my $loop = IO::Async::Loop::IO_Poll->new();
-
- $loop->enable_childmanager;
+ use IO::Async::Loop;
+ my $loop = IO::Async::Loop->new();
 
  ...
 
@@ -71,6 +71,8 @@ sub new
       loop => $loop,
    }, $class;
 
+   weaken( $self->{loop} );
+
    $loop->attach_signal( CHLD => sub { $self->SIGCHLD } );
 
    return $self;
@@ -337,7 +339,9 @@ sub spawn_child
 
    my @setup = defined $setup ? $self->_check_setup_and_canonicise( $setup ) : ();
 
-   pipe( my $readpipe, my $writepipe ) or croak "Cannot pipe() - $!";
+   my $loop = $self->{loop};
+
+   my ( $readpipe, $writepipe ) = $loop->pipepair() or croak "Cannot pipe() - $!";
 
    my $flags = fcntl( $writepipe, F_GETFL, 0 ) or 
       croak "Cannot fcntl(F_GETFL) - $!";
@@ -354,8 +358,6 @@ sub spawn_child
       };
    }
 
-   my $loop = $self->{loop};
-
    my $kid = $loop->detach_child( 
       code => sub {
          # Child
@@ -409,6 +411,14 @@ A non-reference value may be passed as a shortcut, where it would contain the
 name of the operation with no arguments (i.e. for the C<close> and C<keep>
 operations).
 
+=item IO => ARRAY
+
+Shortcut for passing C<fdI<n>>, where I<n> is the fileno of the IO
+reference. In this case, the key must be a reference that implements the
+C<fileno> method. This is mostly useful for
+
+ $handle => 'keep'
+
 =item fdI<n> => IO
 
 A shortcut for the C<dup> case given above.
@@ -425,6 +435,14 @@ Shortcuts for C<fd0>, C<fd1> and C<fd2> respectively.
 
 A reference to a hash to set as the child process's environment.
 
+=item nice => INT
+
+Change the child process's scheduling priority using C<POSIX::nice()>.
+
+=item chdir => STRING
+
+Change the child process's working directory using C<chdir()>.
+
 =back
 
 If no directions for what to do with C<stdin>, C<stdout> and C<stderr> are
@@ -452,6 +470,9 @@ sub _check_setup_and_canonicise
       $key eq "stdout" and $key = "fd1";
       $key eq "stderr" and $key = "fd2";
 
+      # Rewrite other filehandles
+      ref $key and eval { $key->fileno; 1 } and $key = "fd" . $key->fileno;
+
       if( $key =~ m/^fd(\d+)$/ ) {
          my $fd = $1;
          my $ref = ref $value;
@@ -476,6 +497,15 @@ sub _check_setup_and_canonicise
       elsif( $key eq "env" ) {
          ref $value eq "HASH" or croak "Expected HASH reference for 'env' setup key";
       }
+      elsif( $key eq "nice" ) {
+         $value =~ m/^\d+$/ or croak "Expected integer for 'nice' setup key";
+      }
+      elsif( $key eq "chdir" ) {
+         # This isn't a purely watertight test, but it does guard against
+         # silly things like passing a reference - directories such as
+         # ARRAY(0x12345) are unlikely to exist
+         -d $value or croak "Working directory '$value' does not exist";
+      }
       else {
          croak "Unrecognised setup operation '$key'";
       }
@@ -535,8 +565,6 @@ sub _spawn_in_parent
                local $! = $dollarbang;
                $on_exit->( $kid, $exitcode, $!, $dollarat );
             }
-
-            $loop->remove( $self );
          }
 
          return 0;
@@ -652,6 +680,12 @@ sub _spawn_in_child
             elsif( $key eq "env" ) {
                %ENV = %$value;
             }
+            elsif( $key eq "nice" ) {
+               nice( $value ) or die "Cannot nice($value) - $!";
+            }
+            elsif( $key eq "chdir" ) {
+               chdir( $value ) or die "Cannot chdir('$value') - $!";
+            }
          }
       }
 
@@ -769,6 +803,8 @@ sub open_child
       delete $params{setup};
    }
 
+   my $loop = $self->{loop};
+
    foreach my $key ( keys %params ) {
       my $value = $params{$key};
 
@@ -782,7 +818,7 @@ sub open_child
       if( $key =~  m/^fd\d+$/ ) {
          ref $value eq "HASH" or croak "Expected '$orig_key' to be a HASH ref";
 
-         pipe( my ( $pipe_r, $pipe_w ) ) or croak "Unable to pipe() - $!";
+         my ( $pipe_r, $pipe_w ) = $loop->pipepair() or croak "Unable to pipe() - $!";
 
          my ( $myfd, $childfd );
 
@@ -835,8 +871,6 @@ sub open_child
       },
    );
 
-   my $loop = $self->{loop};
-
    $pid = $loop->spawn_child( %subparams, 
       setup => \@setup,
       on_exit => sub {
