@@ -7,14 +7,12 @@ package IO::Async::Connector;
 
 use strict;
 
-our $VERSION = '0.14_2';
+our $VERSION = '0.16.001';
 
 use IO::Async::Notifier;
 
 use POSIX qw( EINPROGRESS );
-use Socket qw( SOL_SOCKET SO_ERROR );
-
-use IO::Socket; # For the actual connections that are created
+use Socket qw( SO_ERROR );
 
 use Carp;
 
@@ -28,10 +26,8 @@ This object is used indirectly via an C<IO::Async::Loop>:
 
  use Socket qw( SOCK_STREAM );
 
- use IO::Async::Loop::IO_Poll;
- my $loop = IO::Async::Loop::IO_Poll->new();
-
- $loop->enable_childmanager;
+ use IO::Async::Loop;
+ my $loop = IO::Async::Loop->new();
 
  $loop->connect(
     host     => "www.example.com",
@@ -103,7 +99,7 @@ sub _get_sock_err
 {
    my ( $sock ) = @_;
 
-   my $err_packed = getsockopt( $sock, SOL_SOCKET, SO_ERROR );
+   my $err_packed = $sock->sockopt( SO_ERROR );
 
    if( defined $err_packed ) {
       my $err = unpack( "I", $err_packed );
@@ -119,10 +115,18 @@ sub _get_sock_err
       return undef;
    }
 
+   my $peername_errno = $!+0;
+   my $peername_errstr = "$!";
+
    # Not connected so we know this ought to fail
    if( read( $sock, my $buff, 1 ) ) {
-      print STDERR "Oops - getpeername() fails but read() returns!\n";
-      # TODO
+      # That was most unexpected. getpeername() fails because we're not
+      # connected, yet read() succeeds.
+      warn "getpeername() fails with $peername_errno ($peername_errstr) but read() is successful\n";
+      warn "Please see http://rt.cpan.org/Ticket/Display.html?id=38382\n";
+
+      $! = $peername_errno;
+      return $!;
    }
 
    return $!;
@@ -133,14 +137,15 @@ sub _connect_addresses
    my $self = shift;
    my ( $addrlist, $on_connected, $on_connect_error, $on_fail ) = @_;
 
+   my $loop = $self->{loop};
+
    my $sock;
    my $address;
 
    while( my $addr = shift @$addrlist ) {
       ( my ( $family, $socktype, $proto ), $address ) = @$addr;
 
-      $sock = IO::Socket->new();
-      $sock->socket( $family, $socktype, $proto ) and last;
+      $sock = $loop->socket( $family, $socktype, $proto ) and last;
 
       undef $sock;
       $on_fail->( "socket", $family, $socktype, $proto, $! ) if $on_fail;
@@ -169,8 +174,6 @@ sub _connect_addresses
 
    # Now we'll set up a Notifier for a one-shot check on it being writable.
 
-   my $loop = $self->{loop};
-
    my $notifier = IO::Async::Notifier->new(
       write_handle => $sock,
 
