@@ -7,13 +7,12 @@ package IO::Async::DetachedCode;
 
 use strict;
 
-our $VERSION = '0.14_2';
+our $VERSION = '0.16.001';
 
 use IO::Async::Stream;
 
 use Carp;
-
-use Socket;
+use Scalar::Util qw( weaken );
 
 use constant LENGTH_OF_I => length( pack( "I", 0 ) );
 
@@ -25,10 +24,8 @@ C<IO::Async::DetachedCode> - execute code asynchronously in child processes
 
 This object is used indirectly via an C<IO::Async::Loop>:
 
- use IO::Async::Loop::IO_Poll;
- my $loop = IO::Async::Loop::IO_Poll->new();
-
- $loop->enable_childmanager;
+ use IO::Async::Loop;
+ my $loop = IO::Async::Loop->new();
 
  my $code = $loop->detach_code(
     code => sub {
@@ -115,10 +112,11 @@ provided to the C<call()> method.
 Optional string, specifies which sort of stream will be used to attach to each
 worker. C<socket> uses only one file descriptor per worker in the parent
 process, but not all systems may be able to use it. If the system does not
-allow C<PF_UNIX> socket pairs, then C<pipe> can be used instead. This will use
+support C<socketpair()>, then C<pipe> can be used instead. This will use
 two file descriptors per worker in the parent process, however.
 
-If not supplied, the C<socket> method is used.
+If not supplied, the underlying Loop's C<pipequad()> method is used, which
+will select an appropriate method. Usually this default will be sufficient.
 
 =item marshaller => STRING: C<flat> or C<storable>
 
@@ -143,6 +141,13 @@ exception. If missing or false, the worker will continue running to process
 more requests. If true, the worker will be shut down. A new worker might be
 constructed by the C<call> method to replace it, if necessary.
 
+=item setup => ARRAY
+
+Optional array reference. Specifies the C<setup> key to pass to the underlying
+C<detach_child> when detaching the code block. If not supplied, a default one
+will be created which just closes C<STDIN> and C<STDOUT>; C<STDERR> will be
+left unaffected.
+
 =back
 
 Since the code block will be called multiple times within the same child
@@ -187,9 +192,9 @@ sub new
       croak "Unrecognised marshaller type '$params{marshaller}'";
    }
 
-   my $streamtype = $params{stream} || "socket";
+   my $streamtype = $params{stream};
 
-   $streamtype eq "socket" or $streamtype eq "pipe" or
+   !defined $streamtype or $streamtype eq "socket" or $streamtype eq "pipe" or
       croak "Unrecognised stream type '$streamtype'";
 
    my $workers = $params{workers} || 1;
@@ -197,6 +202,15 @@ sub new
    # Squash this down to a boolean
    my $exit_on_die =  $params{exit_on_die} ? 1 : 0;
 
+   # Provide a child setup list if one wasn't given
+   my $setup = $params{setup};
+
+   $setup ||= [
+      stdin  => 'close',
+      stdout => 'close',
+      # stderr is kept by default
+   ];
+
    my $self = bless {
       next_id     => 0,
       code        => $code,
@@ -204,6 +218,7 @@ sub new
       streamtype  => $streamtype,
       marshaller  => $marshaller,
       workers     => $workers,
+      setup       => $setup,
       exit_on_die => $exit_on_die,
 
       inners => [],
@@ -220,9 +235,11 @@ sub _detach_child
 {
    my $self = shift;
 
+   my $loop = $self->{loop};
+
    # The inner object needs references to some members of the outer object
    my $inner = {
-      loop           => $self->{loop},
+      loop           => $loop,
       result_handler => {},
       marshaller     => $self->{marshaller},
       busy           => 0,
@@ -231,38 +248,40 @@ sub _detach_child
       exit_on_die    => $self->{exit_on_die},
    };
 
+   weaken( $inner->{loop} );
+
+   # Not required to keep Loop's refcount happy; but does break a cycle here
+   #weaken( $inner->{inners} );
+
    my ( $childread, $mywrite );
    my ( $myread, $childwrite );
 
    my $streamtype = $self->{streamtype};
 
-   if( $streamtype eq "socket" ) {
-      socketpair( my $myend, my $childend, PF_UNIX, SOCK_STREAM, 0 ) or
-         croak "Cannot socketpair(PF_UNIX) - $!";
+   if( !defined $streamtype ) {
+      ( $childread, $mywrite, $myread, $childwrite ) = $loop->pipequad() or
+         croak "Cannot pipequad() - $!";
+   }
+   elsif( $streamtype eq "socket" ) {
+      my ( $myend, $childend ) = $loop->socketpair() or
+         croak "Cannot socketpair() - $!";
 
       $mywrite = $myread = $myend;
       $childwrite = $childread = $childend;
    }
    elsif( $streamtype eq "pipe" ) {
-      pipe( $childread, $mywrite ) or croak "Cannot pipe() - $!";
-      pipe( $myread, $childwrite ) or croak "Cannot pipe() - $!";
+      ( $childread, $mywrite ) = $loop->pipepair() or croak "Cannot pipe() - $!";
+      ( $myread, $childwrite ) = $loop->pipepair() or croak "Cannot pipe() - $!";
    }
 
-   my $loop = $inner->{loop};
-
-   my $fdread  = "fd" . fileno( $childread ); # TODO: Make this neater
-   my $fdwrite = "fd" . fileno( $childwrite );
-
    my $kid = $loop->spawn_child(
       code => sub { 
          $self->_child_loop( $childread, $childwrite, $inner ),
       },
       setup => [
-         stdin  => 'close',
-         stdout => 'close',
-         # stderr is kept by default
-         $fdread  => 'keep',
-         $fdwrite => 'keep',
+         @{ $self->{setup} },
+         $childread  => 'keep',
+         $childwrite => 'keep',
       ],
       on_exit => sub { 
          my ( $pid, $exitcode, undef, undef ) = @_;
@@ -284,6 +303,9 @@ sub _detach_child
 
    $inner->{iostream} = $iostream;
 
+   # Not required to keep Loop's refcount happy; but does break a cycle here
+   #weaken( $inner->{iostream} );
+
    $loop->add( $iostream );
 
    push @{ $self->{inners} }, $inner;
@@ -637,10 +659,6 @@ object.
 
 Dynamic pooling of multiple worker processes, with min/max watermarks.
 
-=item *
-
-Fall back on a pipe pair if socketpair doesn't work.
-
 =back
 
 =head1 NOTES
