@@ -7,10 +7,13 @@ package IO::Async::Loop;
 
 use strict;
 
-our $VERSION = '0.14_2';
+our $VERSION = '0.16.001';
 
 use Carp;
 
+use Socket;
+use IO::Socket;
+
 # Never sleep for more than 1 second if a signal proxy is registered, to avoid
 # a borderline race condition.
 # There is a race condition in perl involving signals interacting with XS code
@@ -21,6 +24,15 @@ use Carp;
 # run when the XS function returns. 
 our $MAX_SIGWAIT_TIME = 1;
 
+# Maybe our calling program will have a suggested hint of a specific Loop
+# class or list of classes to use
+our $LOOP;
+
+# Undocumented; used only by the test scripts.
+# Setting this value true will avoid the IO::Async::Loop::$^O candidate in the
+# magic constructor
+our $LOOP_NO_OS;
+
 BEGIN {
    if ( eval { Time::HiRes::time(); 1 } ) {
       Time::HiRes->import( qw( time ) );
@@ -33,7 +45,25 @@ C<IO::Async::Loop> - core loop of the C<IO::Async> framework
 
 =head1 SYNOPSIS
 
-This module would not be used directly; see the subclasses:
+ use IO::Async::Loop;
+
+ my $loop = IO::Async::Loop->new();
+
+ $loop->add( ... );
+
+ $loop->loop_forever();
+
+=head1 DESCRIPTION
+
+This module provides an abstract class which implements the core loop of the
+C<IO::Async> framework. Its primary purpose is to store a set of
+C<IO::Async::Notifier> objects or subclasses of them. It handles all of the
+lower-level set manipulation actions, and leaves the actual IO readiness 
+testing/notification to the concrete class that implements it. It also
+provides other functionallity such as signal handling, child process managing,
+and timers.
+
+See also the two bundled Loop subclasses:
 
 =over 4
 
@@ -46,16 +76,6 @@ This module would not be used directly; see the subclasses:
 Or other subclasses that may appear on CPAN which are not part of the core
 C<IO::Async> distribution.
 
-=head1 DESCRIPTION
-
-This module provides an abstract class which implements the core loop of the
-C<IO::Async> framework. Its primary purpose is to store a set of
-C<IO::Async::Notifier> objects or subclasses of them. It handles all of the
-lower-level set manipulation actions, and leaves the actual IO readiness 
-testing/notification to the concrete class that implements it. It also
-provides other functionallity such as signal handling, child process managing,
-and timers.
-
 =cut
 
 # Internal constructor used by subclasses
@@ -73,6 +93,87 @@ sub __new
    return $self;
 }
 
+=head1 MAGIC CONSTRUCTOR
+
+=head2 $loop = IO::Async::Loop->new()
+
+This function attempts to find a good subclass to use, then calls its
+constructor. It works by making a list of likely candidate classes, then
+trying each one in turn, C<require>ing the module then calling its C<new>
+method. If either of these operations fails, the next subclass is tried. If
+no class was successful, then an exception is thrown.
+
+The list of candidates is formed from the following choices, in this order:
+
+=over 4
+
+=item * $ENV{IO_ASYNC_LOOP}
+
+If this environment variable is set, it should contain a comma-separated list
+of subclass names. These names may or may not be fully-qualified; if a name
+does not contain C<::> then it will have C<IO::Async::Loop::> prepended to it.
+This allows the end-user to specify a particular choice to fit the needs of
+his use of a program using C<IO::Async>.
+
+=item * $IO::Async::Loop::LOOP
+
+If this scalar is set, it should contain a comma-separated list of subclass
+names. These may or may not be fully-qualified, as with the above case. This
+allows a program author to suggest a loop module to use.
+
+In cases where the module subclass is a hard requirement, such as GTK programs
+using C<Glib>, it would be better to use the module specifically and invoke
+its constructor directly.
+
+=item * $^O
+
+The module called C<IO::Async::Loop::$^O> is tried next. This allows specific
+OSes, such as the ever-tricky C<MSWin32>, to provide an implementation that
+might be more efficient than the generic ones, or even work at all.
+
+=item * IO_Poll and Select
+
+Finally, if no other choice has been made by now, the built-in C<IO_Poll>
+module is chosen. This should always work, but in case it doesn't, the
+C<Select> module will be chosen afterwards as a last-case attempt. If this
+also fails, then the magic constructor itself will throw an exception.
+
+=back
+
+=cut
+
+sub new
+{
+   shift;  # We're going to ignore the class name actually given
+
+   my @candidates;
+
+   push @candidates, split( m/,/, $ENV{IO_ASYNC_LOOP} ) if defined $ENV{IO_ASYNC_LOOP};
+
+   push @candidates, split( m/,/, $LOOP ) if defined $LOOP;
+
+   push @candidates, "$^O" unless $LOOP_NO_OS;
+
+   push @candidates, "IO_Poll", "Select";
+
+   $_ =~ m/::/ or $_ = "IO::Async::Loop::$_" for @candidates;
+
+   foreach my $class ( @candidates ) {
+      ( my $file = "$class.pm" ) =~ s{::}{/}g;
+
+      eval { require $file } or next;
+
+      my $self;
+      $self = eval { $class->new } and return $self;
+
+      # Oh dear. We've loaded the code OK but for some reason the constructor
+      # wasn't happy. Being polite we ought really to unload the file again,
+      # but perl doesn't actually provide us a way to do this.
+   }
+
+   croak "Cannot find a suitable candidate class";
+}
+
 =head1 METHODS
 
 =cut
@@ -205,6 +306,12 @@ sub __new_feature
    ( my $filename = "$classname.pm" ) =~ s{::}{/}g;
    require $filename;
 
+   # These features aren't supposed to be "user visible", so if methods called
+   # on it carp or croak, the shortmess line ought to skip IO::Async::Loop and
+   # go on report its caller. To make this work, add the feature class to our
+   # @CARP_NOT list.
+   push our(@CARP_NOT), $classname;
+
    return $classname->new( loop => $self );
 }
 
@@ -224,6 +331,12 @@ A CODE reference to the handling function.
 
 =back
 
+Attaching to C<SIGCHLD> is not recommended because of the way all child
+processes use it to report their termination. Instead, the C<watch_child>
+method should be used to watch for termination of a given child process. A
+warning will be printed if C<SIGCHLD> is passed here, but in future versions
+of C<IO::Async> this behaviour may be disallowed altogether.
+
 See also L<POSIX> for the C<SIGI<name>> constants.
 
 =cut
@@ -259,14 +372,20 @@ sub detach_signal
    my $sigproxy = $self->{sigproxy} ||= $self->__new_feature( "IO::Async::SignalProxy" );
    $sigproxy->detach( $signal );
 
-   # TODO: Consider "refcount" signals and cleanup if zero. How do we know if
-   # anyone else has a reference to the signal proxy though? Tricky...
+   if( !$sigproxy->signals ) {
+      $self->remove( $sigproxy );
+      undef $sigproxy;
+      undef $self->{sigproxy};
+   }
 }
 
 =head2 $loop->enable_childmanager
 
 This method enables the child manager, which allows use of the
-C<watch_child()>, C<detach_child()> and C<spawn_child()> methods.
+C<watch_child()> methods without a race condition.
+
+The child manager will be automatically enabled if required; so this method
+does not need to be explicitly called for other C<*_child()> methods.
 
 =cut
 
@@ -274,10 +393,7 @@ sub enable_childmanager
 {
    my $self = shift;
 
-   defined $self->{childmanager} and
-      croak "ChildManager already enabled for this loop";
-
-   $self->{childmanager} = $self->__new_feature( "IO::Async::ChildManager" );
+   $self->{childmanager} ||= $self->__new_feature( "IO::Async::ChildManager" );
 }
 
 =head2 $loop->disable_childmanager
@@ -290,19 +406,22 @@ sub disable_childmanager
 {
    my $self = shift;
 
-   defined $self->{childmanager} or
-      croak "ChildManager not enabled for this loop";
-
-   my $childmanager = $self->{childmanager};
-   $childmanager->disable;
-
-   undef $self->{childmanager};
+   if( my $childmanager = $self->{childmanager} ) {
+      $childmanager->disable;
+      undef $self->{childmanager};
+   }
 }
 
 =head2 $loop->watch_child( $pid, $code )
 
 This method adds a new handler for the termination of the given child PID.
 
+Because the process represented by C<$pid> may already have exited by the time
+this method is called, the child manager should already have been enabled
+before it was C<fork()>ed, by calling C<enable_childmanager>. If this is not
+done, then a C<SIGCHLD> signal may have been missed, and the exit of this
+child process will not be reported.
+
 =cut
 
 sub watch_child
@@ -310,8 +429,8 @@ sub watch_child
    my $self = shift;
    my ( $kid, $code ) = @_;
 
-   my $childmanager = $self->{childmanager} or
-      croak "ChildManager not enabled in Loop";
+   my $childmanager = $self->{childmanager} ||=
+      $self->__new_feature( "IO::Async::ChildManager" );
 
    $childmanager->watch_child( $kid, $code );
 }
@@ -329,8 +448,8 @@ sub detach_child
    my $self = shift;
    my %params = @_;
 
-   my $childmanager = $self->{childmanager} or
-      croak "ChildManager not enabled in Loop";
+   my $childmanager = $self->{childmanager} ||=
+      $self->__new_feature( "IO::Async::ChildManager" );
 
    $childmanager->detach_child( %params );
 }
@@ -369,8 +488,8 @@ sub spawn_child
    my $self = shift;
    my %params = @_;
 
-   my $childmanager = $self->{childmanager} or
-      croak "ChildManager not enabled in Loop";
+   my $childmanager = $self->{childmanager} ||=
+      $self->__new_feature( "IO::Async::ChildManager" );
 
    $childmanager->spawn_child( %params );
 }
@@ -388,8 +507,8 @@ sub open_child
    my $self = shift;
    my %params = @_;
 
-   my $childmanager = $self->{childmanager} or
-      croak "ChildManager not enabled in Loop";
+   my $childmanager = $self->{childmanager} ||=
+      $self->__new_feature( "IO::Async::ChildManager" );
 
    $childmanager->open_child( %params );
 }
@@ -408,8 +527,8 @@ sub run_child
    my $self = shift;
    my %params = @_;
 
-   my $childmanager = $self->{childmanager} or
-      croak "ChildManager not enabled in Loop";
+   my $childmanager = $self->{childmanager} ||=
+      $self->__new_feature( "IO::Async::ChildManager" );
 
    $childmanager->run_child( %params );
 }
@@ -436,11 +555,33 @@ sub _adjust_timeout
    if( $timer_delay < 0 ) {
       $$timeref = 0;
    }
-   elsif( $timer_delay < \$timeref ) {
+   elsif( !defined $$timeref or $timer_delay < $$timeref ) {
       $$timeref = $timer_delay;
    }
 }
 
+# For subclasses to call
+sub _build_time
+{
+   my $self = shift;
+   my %params = @_;
+
+   my $time;
+   if( exists $params{time} ) {
+      $time = $params{time};
+   }
+   elsif( exists $params{delay} ) {
+      my $now = exists $params{now} ? $params{now} : time();
+
+      $time = $now + $params{delay};
+   }
+   else {
+      croak "Expected either 'time' or 'delay' keys";
+   }
+
+   return $time;
+}
+
 =head2 $id = $loop->enqueue_timer( %params )
 
 This method installs a callback which will be called at the specified time.
@@ -490,6 +631,8 @@ sub enqueue_timer
 
    my $timequeue = $self->{timequeue} ||= $self->__new_feature( "IO::Async::TimeQueue" );
 
+   $params{time} = $self->_build_time( %params );
+
    $timequeue->enqueue( %params );
 }
 
@@ -509,6 +652,32 @@ sub cancel_timer
    $timequeue->cancel( $id );
 }
 
+=head2 $newid = $loop->requeue_timer( $id, %params )
+
+Reschedule an existing timer, moving it to a new time. The old timer is
+removed and will not be invoked.
+
+The C<%params> hash takes the same keys as C<enqueue_timer()>, except for the
+C<code> argument.
+
+The requeue operation may be implemented as a cancel + enqueue, which may
+mean the ID changes. Be sure to store the returned C<$newid> value if it is
+required.
+
+=cut
+
+sub requeue_timer
+{
+   my $self = shift;
+   my ( $id, %params ) = @_;
+
+   my $timequeue = $self->{timequeue} ||= $self->__new_feature( "IO::Async::TimeQueue" );
+
+   $params{time} = $self->_build_time( %params );
+
+   $timequeue->requeue( $id, %params );
+}
+
 =head2 $loop->resolve( %params )
 
 This method performs a single name resolution operation. It uses an
@@ -620,6 +789,105 @@ sub loop_stop
    $self->{still_looping} = 0;
 }
 
+=head1 OS ABSTRACTIONS
+
+Because the Magic Constructor searches for OS-specific subclasses of the Loop,
+several abstractions of OS services are provided, in case specific OSes need
+to give different implementations on that OS.
+
+=cut
+
+# This one isn't documented because it's not really overridable. It's largely
+# here just for completeness
+sub socket
+{
+   my $self = shift;
+   my ( $family, $socktype, $proto ) = @_;
+
+   croak "Cannot create a new socket() without a family" unless $family;
+
+   # SOCK_STREAM is the most likely
+   defined $socktype or $socktype = SOCK_STREAM;
+
+   defined $proto or $proto = 0;
+
+   return IO::Socket->new->socket( $family, $socktype, $proto );
+}
+
+=head2 ( $S1, $S2 ) = $loop->socketpair( $family, $socktype, $proto )
+
+An abstraction of the C<socketpair()> syscall, where any argument may be
+missing (or given as C<undef>).
+
+If C<$family> is not provided, a suitable value will be provided by the OS
+(likely C<AF_UNIX> on POSIX-based platforms). If C<$socktype> is not provided,
+then C<SOCK_STREAM> will be used.
+
+=cut
+
+sub socketpair
+{
+   my $self = shift;
+   my ( $family, $socktype, $proto ) = @_;
+
+   # PF_UNSPEC and undef are both false
+   $family ||= AF_UNIX;
+
+   # SOCK_STREAM is the most likely
+   defined $socktype or $socktype = SOCK_STREAM;
+
+   defined $proto or $proto = 0;
+
+   return IO::Socket->new->socketpair( $family, $socktype, $proto );
+}
+
+=head2 ( $rd, $wr ) = $loop->pipepair()
+
+An abstraction of the C<pipe()> syscall, which returns the two new handles.
+
+=cut
+
+sub pipepair
+{
+   my $self = shift;
+
+   pipe( my ( $rd, $wr ) ) or return;
+   return ( $rd, $wr );
+}
+
+=head2 ( $rdA, $wrA, $rdB, $wrB ) = $loop->pipequad()
+
+This method is intended for creating two pairs of filehandles that are linked
+together, suitable for passing as the STDIN/STDOUT pair to a child process.
+After this function returns, C<$rdA> and C<$wrA> will be a linked pair, as
+will C<$rdB> and C<$wrB>.
+
+On platforms that support C<socketpair()>, this implementation will be
+preferred, in which case C<$rdA> and C<$wrB> will actually be the same
+filehandle, as will C<$rdB> and C<$wrA>. This saves a file descriptor in the
+parent process.
+
+When creating a C<IO::Async::Stream> or subclass of it, the C<read_handle>
+and C<write_handle> parameters should always be used.
+
+=cut
+
+sub pipequad
+{
+   my $self = shift;
+
+   # Prefer socketpair()
+   if( my ( $S1, $S2 ) = $self->socketpair() ) {
+      return ( $S1, $S2, $S2, $S1 );
+   }
+
+   # Can't do that, fallback on pipes
+   my ( $rdA, $wrA ) = $self->pipepair() or return;
+   my ( $rdB, $wrB ) = $self->pipepair() or return;
+
+   return ( $rdA, $wrA, $rdB, $wrB );
+}
+
 # Keep perl happy; keep Britain tidy
 1;
 
