@@ -1,15 +1,16 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2006,2007 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2006-2008 -- leonerd@leonerd.org.uk
 
 package IO::Async::Notifier;
 
 use strict;
 
-our $VERSION = '0.14_2';
+our $VERSION = '0.16.001';
 
 use Carp;
+use Scalar::Util qw( weaken );
 
 =head1 NAME
 
@@ -20,8 +21,8 @@ C<IO::Async::Notifier> - event callbacks for a non-blocking file descriptor
  use IO::Socket::INET;
  use IO::Async::Notifier;
 
- use IO::Async::Loop::IO_Poll;
- my $loop = IO::Async::Loop::IO_Poll->new();
+ use IO::Async::Loop;
+ my $loop = IO::Async::Loop->new();
 
  my $socket = IO::Socket::INET->new( LocalPort => 1234, Listen => 1 );
 
@@ -128,6 +129,12 @@ is a subclass that overrides the method. I.e. if only a C<read_handle> is
 given, then C<on_write_ready> can be absent. If C<handle> is used as a
 shortcut, then both read and write-ready callbacks or methods are required.
 
+If no IO handles are provided at construction time, the object is still
+created but will not yet be fully-functional as a Notifier. IO handles can
+be assigned later using the C<set_handle> or C<set_handles> methods. This may
+be useful when constructing an object to represent a network connection,
+before the C<connect()> has actually been performed yet.
+
 =cut
 
 sub new
@@ -166,9 +173,6 @@ sub new
       $read_handle  = $handle;
       $write_handle = $handle;
    }
-   else {
-      croak "Expected either 'handle' or 'read_handle' and 'write_handle' keys";
-   }
 
    if( defined $read_handle ) {
       if( !$params{on_read_ready} and $class->can( 'on_read_ready' ) == \&on_read_ready ) {
@@ -186,7 +190,8 @@ sub new
    my $self = bless {
       read_handle     => $read_handle,
       write_handle    => $write_handle,
-      want_writeready => $params{want_writeready} || 0,
+      want_readready  => 0,
+      want_writeready => 0,
       children        => [],
       parent          => undef,
    }, $class;
@@ -197,7 +202,7 @@ sub new
 
    # Slightly asymmetric
    $self->want_readready( defined $read_handle );
-   $self->want_writeready( $self->{want_writeready} );
+   $self->want_writeready( $params{want_writeready} || 0 );
 
    return $self;
 }
@@ -206,6 +211,74 @@ sub new
 
 =cut
 
+=head2 $notifier->set_handles( %params )
+
+This method stores new reading or writing handles in the object, as if they
+had been passed as the C<read_handle> or C<write_handle> arguments to the
+constructor. The C<%params> hash takes the following keys:
+
+=over 8
+
+=item read_handle => IO
+
+A new IO handle for reading, or C<undef> to remove the old one.
+
+=item write_handle => IO
+
+A new IO handle for writing, or C<undef> to remove the old one.
+
+=back
+
+=cut
+
+sub set_handles
+{
+   my $self = shift;
+   my %params = @_;
+
+   if( defined $params{read_handle} ) {
+      unless( defined eval { $params{read_handle}->fileno } ) {
+         croak 'Expected that read_handle can fileno()';
+      }
+   }
+
+   if( defined $params{write_handle} ) {
+      unless( defined eval { $params{write_handle}->fileno } ) {
+         croak 'Expected that write_handle can fileno()';
+      }
+   }
+
+   if( exists $params{read_handle} ) {
+      $self->{read_handle} = $params{read_handle};
+
+      # Register interest in readability with the underlying loop
+      $self->want_readready( defined $self->{read_handle} );
+   }
+
+   if( exists $params{write_handle} ) {
+      $self->{write_handle} = $params{write_handle};
+   }
+}
+
+=head2 $notifier->set_handle( $handle )
+
+Shortcut for
+
+ $notifier->set_handles( read_handle => $handle, write_handle => $handle )
+
+=cut
+
+sub set_handle
+{
+   my $self = shift;
+   my ( $handle ) = @_;
+
+   $self->set_handles(
+      read_handle  => $handle,
+      write_handle => $handle,
+   );
+}
+
 =head2 $notifier->close
 
 This method calls C<close> on the underlying IO handles. This method will will
@@ -217,9 +290,12 @@ sub close
 {
    my $self = shift;
 
-   $self->{on_closed}->( $self ) if $self->{on_closed};
+   return unless defined $self->read_handle or defined $self->write_handle;
 
-   if( my $loop = $self->{loop} ) {
+   if( my $parent = $self->{parent} ) {
+      $parent->remove_child( $self );
+   }
+   elsif( my $loop = $self->{loop} ) {
       $loop->remove( $self );
    }
 
@@ -228,6 +304,8 @@ sub close
 
    my $write_handle = delete $self->{write_handle};
    $write_handle->close if defined $write_handle and ( not defined $read_handle or $write_handle != $read_handle );
+
+   $self->{on_closed}->( $self ) if $self->{on_closed};
 }
 
 =head2 $handle = $notifier->read_handle
@@ -292,6 +370,7 @@ sub __set_loop
    my $self = shift;
    my ( $loop ) = @_;
    $self->{loop} = $loop;
+   weaken( $self->{loop} ); # To avoid a cycle
 }
 
 =head2 $value = $notifier->want_readready
@@ -314,6 +393,9 @@ sub want_readready
    if( @_ ) {
       my ( $new ) = @_;
 
+      $new = $new ?1:0; # Squash to boolean
+      return $new if $new == $self->{want_readready};
+
       if( $new ) {
          defined $self->read_handle or
             croak 'Cannot want_readready in a Notifier with no read_handle';
@@ -339,6 +421,9 @@ sub want_writeready
    if( @_ ) {
       my ( $new ) = @_;
 
+      $new = $new ?1:0; # Squash to boolean
+      return $new if $new == $self->{want_writeready};
+
       if( $new ) {
          defined $self->write_handle or
             croak 'Cannot want_writeready in a Notifier with no write_handle';
