@@ -7,7 +7,7 @@ package IO::Async::Sequencer;
 
 use strict;
 
-our $VERSION = '0.14_2';
+our $VERSION = '0.16.001';
 
 use base qw( IO::Async::Stream );
 
@@ -21,8 +21,8 @@ C<IO::Async::Sequencer> - handle a serial pipeline of requests / responses (EXPE
 
 When used as a client:
 
- use IO::Async::Loop::IO_Poll;
- my $loop = IO::Async::Loop::IO_Poll->new();
+ use IO::Async::Loop;
+ my $loop = IO::Async::Loop->new();
 
  my $sock = ...
 
@@ -202,9 +202,10 @@ sub new
    my $class = shift;
    my %params = @_;
 
-   defined $params{handle} or
-      ( defined $params{read_handle} and defined $params{write_handle} ) or
-      croak "Sequencer requires both a reading and a writing handle"; # TODO: this message sucks
+   if(  defined $params{read_handle} and !defined $params{write_handle} or
+       !defined $params{read_handle} and  defined $params{write_handle} ) {
+      croak "Sequencer requires both a reading and a writing handle";
+   }
 
    my $on_read = delete $params{on_read};
    ref $on_read eq "CODE" or croak "Expected 'on_read' as a CODE reference";
