@@ -1,13 +1,13 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2007 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2007,2008 -- leonerd@leonerd.org.uk
 
 package IO::Async::SignalProxy;
 
 use strict;
 
-our $VERSION = '0.14_2';
+our $VERSION = '0.16.001';
 
 use base qw( IO::Async::Notifier );
 
@@ -44,7 +44,7 @@ sub new
 
    my $loop = delete $params{loop} or croak "Expected a 'loop'";
 
-   pipe( my ( $reader, $writer ) ) or croak "Cannot pipe() - $!";
+   my ( $reader, $writer ) = $loop->pipepair() or croak "Cannot pipe() - $!";
 
    $reader->blocking( 0 );
    $writer->blocking( 0 );
@@ -128,6 +128,12 @@ sub on_read_ready
 
    foreach( @caught_signals ) {
       my $callback = $self->{callbacks}->{$_};
+      # This shouldn't fail, but I've seen what looked like a race condition
+      # failure on a Solaris 8 + perl 5.6.1 machine here. This message should
+      # at least help find the problem
+      defined $callback or croak "Undefined callback for signal $_";
+      ref $callback eq "CODE" or croak "Callback for signal $_ is not a CODE ref";
+
       $callback->();
    }
 }
@@ -137,11 +143,19 @@ sub attach
    my $self = shift;
    my ( $signal, $code ) = @_;
 
+   if( $signal eq "CHLD" ) {
+      # We make special exception to allow the ChildManager to do this
+      caller(1) eq "IO::Async::ChildManager" or
+         carp "Attaching to SIGCHLD is not advised - use the IO::Async::ChildManager instead";
+   }
+
    exists $SIG{$signal} or croak "Unrecognised signal name $signal";
 
    # Don't allow anyone to trash an existing signal handler
    !defined $SIG{$signal} or !ref $SIG{$signal} or croak "Cannot override signal handler for $signal";
 
+   ref $code eq "CODE" or croak 'Expected $code as a CODE reference';
+
    $self->{callbacks}->{$signal} = $code;
 
    $self->{restore_SIG}->{$signal} = $SIG{$signal};
@@ -191,6 +205,13 @@ sub detach
    $SIG{$signal} = $restore_SIG->{$signal} || 'DEFAULT';
 
    delete $restore_SIG->{$signal};
+   delete $self->{callbacks}->{$signal};
+}
+
+sub signals
+{
+   my $self = shift;
+   return keys %{ $self->{callbacks} };
 }
 
 # Keep perl happy; keep Britain tidy
