@@ -1,17 +1,20 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2006,2007 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2006-2008 -- leonerd@leonerd.org.uk
 
 package IO::Async::TimeQueue;
 
 use strict;
 
-our $VERSION = '0.14_2';
+our $VERSION = '0.16.001';
 
 use Carp;
+
 use Heap::Fibonacci;
 
+use IO::Async::TimeQueue::Elem;
+
 BEGIN {
    if ( eval { Time::HiRes::time(); 1 } ) {
       Time::HiRes->import( qw( time ) );
@@ -88,22 +91,12 @@ The C<%params> takes the following keys:
 
 The absolute system timestamp to run the event.
 
-=item delay => NUM
-
-The delay after now at which to run the event.
-
-=item now => NUM
-
-The time to consider as now; defaults to C<time()> if not specified.
-
 =item code => CODE
 
 CODE reference to the callback function to run at the allotted time.
 
 =back
 
-Either C<time> or C<delay> must be specified.
-
 =cut
 
 sub enqueue
@@ -111,22 +104,12 @@ sub enqueue
    my $self = shift;
    my ( %params ) = @_;
 
-   my $time;
-   if( exists $params{time} ) {
-      $time = delete $params{time};
-   }
-   elsif( exists $params{delay} ) {
-      my $now = exists $params{now} ? $params{now} : time();
-
-      $time = $now + delete $params{delay};
-   }
-   else {
-      croak "Expected either 'time' or 'delay' keys";
-   }
-
    my $code = delete $params{code};
    ref $code eq "CODE" or croak "Expected 'code' to be a CODE reference";
 
+   defined $params{time} or croak "Expected 'time'";
+   my $time = $params{time};
+
    my $heap = $self->{heap};
 
    my $elem = IO::Async::TimeQueue::Elem->new( $time, $code );
@@ -150,6 +133,39 @@ sub cancel
    $heap->delete( $id );
 }
 
+=head2 $newid = $queue->requeue( $id, %params )
+
+Reschedule an existing timer, moving it to a new time. The old timer is
+removed and will not be invoked.
+
+The C<%params> hash takes the same keys as C<enqueue()>, except for the
+C<code> argument.
+
+The requeue operation may be implemented as a cancel + enqueue, which may
+mean the ID changes. Be sure to store the returned C<$newid> value if it is
+required.
+
+=cut
+
+sub requeue
+{
+   my $self = shift;
+   my ( $id, %params ) = @_;
+
+   defined $params{time} or croak "Expected 'time'";
+   my $time = $params{time};
+
+   my $heap = $self->{heap};
+   my $elem = $heap->delete( $id );
+   defined $elem or croak "No such enqueued timer";
+
+   $elem->time( $time );
+
+   $heap->add( $elem );
+
+   return $elem;
+}
+
 =head2 $count = $queue->fire( %params )
 
 Call all the event callbacks that should have run by now. The number of
@@ -193,92 +209,6 @@ sub fire
 # Keep perl happy; keep Britain tidy
 1;
 
-package IO::Async::TimeQueue::Elem;
-
-use strict;
-use base qw( Heap::Elem );
-
-# The internal implementation of Heap::Elem changed at 0.80 to be an ARRAY
-# typed object, with a 'val' accessor for the user data, where before it's a
-# plain HASH to be accessed directly. We therefore supply two sets of methods
-# here, and set up the right ones depending on the version.
-
-sub new_HASH
-{
-   my $self = shift;
-   my $class = ref $self || $self;
-
-   my ( $time, $code ) = @_;
-
-   my $new = $class->SUPER::new();
-
-   $new->{time} = $time;
-   $new->{code} = $code;
-
-   return $new;
-}
-
-sub time_HASH
-{
-   my $self = shift;
-   return $self->{time};
-}
-
-sub code_HASH
-{
-   my $self = shift;
-   return $self->{code};
-}
-
-sub new_VAL
-{
-   my $self = shift;
-   my $class = ref $self || $self;
-
-   my ( $time, $code ) = @_;
-
-   my $new = $class->SUPER::new(
-      time => $time,
-      code => $code,
-   );
-
-   return $new;
-}
-
-sub time_VAL
-{
-   my $self = shift;
-   return $self->val->{time};
-}
-
-sub code_VAL
-{
-   my $self = shift;
-   return $self->val->{code};
-}
-
-if( $Heap::Elem::VERSION < 0.80 ) {
-  *new  = \&new_HASH;
-  *time = \&time_HASH;
-  *code = \&code_HASH;
-}
-else {
-  *new  = \&new_VAL;
-  *time = \&time_VAL;
-  *code = \&code_VAL;
-}
-
-# This only uses methods so is transparent to HASH or ARRAY
-sub cmp
-{
-   my $self = shift;
-   my $other = shift;
-
-   $self->time <=> $other->time;
-}
-
-1;
-
 __END__
 
 =head1 AUTHOR
